import { store } from "@/store/store";

let hovering = false;
let timeoutId = -1;

export function addFocusTrack(
  groupNumber: number | undefined,
  mainNumber: number | undefined,
  subNumber: string | undefined,
  isMark?: boolean | undefined // 是否是评卷，评卷时要考虑标记删除
) {
  hovering = true;

  timeoutId = setTimeout(() => {
    if (hovering) {
      _addFocusTrack(groupNumber, mainNumber, subNumber, isMark);
    }
  }, 200);
}

function _addFocusTrack(
  groupNumber: number | undefined,
  mainNumber: number | undefined,
  subNumber: string | undefined,
  isMark: boolean | undefined // 是否是评卷，评卷时要考虑标记删除
) {
  store.focusTracks.splice(0);

  if (groupNumber) {
    store.currentTask?.questionList
      ?.filter((q) => q.groupNumber === groupNumber)
      ?.map((q) => q.trackList)
      .flat()
      .forEach((t) => {
        // 回评时，如果没被删除
        const shouldAdd = isMark
          ? store.currentTask?.markResult.trackList.includes(t)
          : true;
        if (shouldAdd) store.focusTracks.push(t);
      });
  } else {
    store.currentTask?.questionList
      ?.map((q) => q.trackList)
      .flat()
      .filter((t) => {
        if (mainNumber) {
          return t.mainNumber === mainNumber && t.subNumber === subNumber;
        } else {
          return false;
        }
      })
      .forEach((t) => {
        // 回评时，如果没被删除
        const shouldAdd = isMark
          ? store.currentTask?.markResult.trackList.includes(t)
          : true;
        if (shouldAdd) store.focusTracks.push(t);
      });
  }
  // console.log(store.focusTracks);
}

export function removeFocusTrack() {
  hovering = false;
  clearTimeout(timeoutId);
  store.focusTracks.splice(0);
}
